
; SPI with a 26LC160 EEPROM
; Slower than hardware version.

; This test program demonstrates SPI interfacing to the EEPROM
; Two bytes are written and then read back to verify.


#picaxe 28x1
;#com 4

' Input / Output pin setup

'24LC160 pins
'1 = CS 	= PICAXE output7
'2 = SO 	= PICAXE input4
'3 = WP 	= +5V
'4 = Vss	= 0V
'5 = SI	= PICAXE output5
'6 = SCK	= PICAXE output6
'7 = HOLD 	= +5V
'8 = Vdd  	= +5V
'remember PICAXE input connects to EEPROM output and vice versa!


symbol cs = 7	' chip select is PICAXE output 7

' 25LC160 Commands
symbol WRSR = 1	' write status register
symbol WR   = 2	' write
symbol RD   = 3	' read
symbol WRDI = 4	' write disable
symbol RDSR = 5	' read status register
symbol WREN = 6	' write enable

'Variables
symbol add_low  	= b0	' address low byte
symbol add_high 	= b1	' address high byte
symbol test_data1 = b2	' dummy test data 1
symbol test_data2 = b3	' dummy test data 2
symbol SR_Value1  = b4	' read status regsietr 1
symbol SR_Value2  = b5	' read status register again
Symbol RD_Value1 	= b6	' read data byte 1
Symbol RD_Value2 	= b7	' read data byte 2


	'set cs high and wait a while
init:
	high cs	'chip select high
	pause 50	' wait a while

	'load some test values
	add_high = 0
	add_low = 25
	test_data1 = 55
	test_data2 = 99
	
main:
	
	; write enable
	low cs
	spiout 6,5,MSBFIRST_L,(WREN,2)
	high cs
	
	; write status register to 0 to remove block protect
	low cs
	spiout 6,5,MSBFIRST_L,(WRSR,0)
	high cs
	pause 5	; wait write time
	
	; write enable
	low cs
	spiout 6,5,MSBFIRST_L,(WREN)
	high cs
	
	; read status register to see if it has worked
	; we expect this value to now be 2
	low cs
	spiout 6,5,MSBFIRST_L,(RDSR)
	spiin 6,4,MSBPRE_L,(SR_Value1)
	high cs
	
	; write
	low cs
	spiout 6,5,MSBFIRST_L,(WR,add_high,add_low,test_data1,test_data2)
	high cs
	pause 5	; wait write time 5ms
	
	;read
	low cs
	spiout 6,5,MSBFIRST_L,(RD,add_high, add_low)
	spiin 6,4,MSBPRE_L,(RD_Value1, RD_Value2)
	high cs
	
	; write enable
	low cs
	spiout 6,5,MSBFIRST_L,(WREN)
	high cs
	
	; read status register to see if it has worked
	; we expect this value to now be 2
	low cs
	spiout 6,5,MSBFIRST_L,(RDSR)
	spiin 6,4,MSBPRE_L,(SR_Value2)
	high cs
	
	; write disable the chip
	low cs
	spiout 6,5,MSBFIRST_L,(WRDI)
	high cs

	'display values on screen and then loop
	debug
	pause 1000
	
	'increment test data
	inc test_data1
	inc test_data2
	
	goto main
	

