'Written on 27/04/05 by Ron Russo

'This program simulates pulling a led on a analogue clock, anticlockwise from 60
'and then leaves it on at the right second, therefore creating a curved bar
'of light from 1 to this second.(Similar)
'The Second Hand points to the next led on the Bar as it gets longer (not Similar).
'The single Led chase pulled along from 60 is eavenly timed to end its journey to the right spot
'in a fixed time of one second.(Similar)

'The PICAXE-08 gets a pulse every second from the coil outputs of an analogue clock
'The clock is used intact but runs from a voltage divider from the 5Volt supply.
'The PICAXE-08 does the calculations for the relevent second
'then manipulates the shift registers Data, Clock and MemClear lines to
'produce the LED Light sequence.

'This complicated task is achieved by using two banks of 74LS164 ShiftRegisters.
'Registers (CHASE SR1 to SR8) manage the LED CHASE from 60 to THE RELEVENT SECOND.
'and Registers (BAR SR9 to SR16) manage the BAR OF LIGHT from 1 to THE RELEVENT SECOND
'The outputs of the two banks of registers are isolated by diodes to each LED.
'Placement of components and layout makes it unnecessary to have a Printed Circuit board

'Using the diodes as in the circuit and a regulated 5Volt power supply,
'LED current limiting resistors are unnecessary as the ShiftRegs self limit the
'current through the leds to 10mA for each output .(Use same manufacturer for all ShiftRegs
'for similar LED light output.)
'This current may vary with different brand LEDs and Shift Regs but will be around 10mA. (This is ample as
'the leds are not pulsed and it can be seen from 50 paces in any light conditions).
'Power consumption starts at 230mA and increases to 860mA as the bar of light grows
'so a decent regulated 5volt supply capable of at least 1.2Amp is necessary for a LED drive of 10mA.

'PROGRAM

Symbol Sec = b13              'Seconds counter
Symbol Leds = b12             'Led Chase counter
Symbol TmLength = w0    	'Duration of pulsout to slow down led chase to suit relevent second
Symbol LedChase = 0     	'Pin0
Symbol NewSec = 1             'Pin1
Symbol NewMin = 2             'Pin2
Symbol ChaseData = 4   		'Pin4 (Chase Data pulse output) & (Reset Button input for Synchronization)


MAIN:
                pause 100                       ' Pause for a while, let micro settle
                high NewMin                     ' Set ShiftRegs BAR(SR9-SR16) MRCLEAR line high as a low is used to clear 74164 ShiftRegisters.
                goto RES_MIN                    ' Go and Initilise for start of new Minute

LOOP:           input 4                         ' Set Pin4 as input to check if Sync button is pressed
                if pin4 = 1 then RES_MIN        ' If Sync button is pressed then initialise for new Minute
                if pin3 = 1 then NXT_SEC    	' Else if input pin3 is Low, there is a sec pulse, one second has elapsed so start a New_Sec
                                                ' NOTE:Some analogue clocks have neg going pulses rather than pos going pulses(Change to suit)
                goto LOOP                       ' else if input pin3 is still low waste some time, a new pulse is coming soon.
                        

NXT_SEC:        output 4                        ' Set Pin 4 as Output to allow sending of ChaseData to CHASE(SR1-SR8) ShiftRegs
                if Sec = 0 then RES_MIN         ' If Min is up then go and reset Minute.    (Count is backwards from 60)
                Sec = Sec - 1                   ' else One Sec has elapsed so decrement Seconds
                TmLength = 61 - Sec             ' This is first step to compute time length each led must be ON for relevent second.
                TmLength = TmLength * 90        ' Then Mult by optimum factor so that selected leds finish just before second is up.
                                                ' (90) is optimum for the PICAXE-08 that I am using. This number can be changed to suit.
        
CHASE:  	    high ChaseData          		' Set DATA for chase high for one clock cycle at beginning of chase.CHASE(SR1toSR8)
                for Leds = Sec to 0 step -1     ' Loop and count back Leds starting from relevent second of the Minute to Zero.
                  pulsout LedChase, TmLength 	' Pulse with right duration to increment CLOCK of CHASE(SR1-SR8) ShiftRegs start at led 60.
                  low ChaseData                 ' After first clock pulse, set DATA low for duration of chase.
                next Leds                       ' Do rest of leds until this sec led chase is finished.
                low NewSec                      ' Send Neg going pulse to reset CHASE(SR1-SR8)ShiftRegs this will also increment (Bank1 SR9-SR16) Clock
                pause 1                         ' pause 1 millsec
                high NewSec                     ' Set MRCLEAR line high ready for next Reset
                goto LOOP                    	' Check input pins for next second pulse or for reset button
                
RES_MIN:        Sec = 60                       	' Must be a Start Up or Min is up or Synchro button is pressed so reset Seconds to 60
                low NewMin                      ' Send a Neg going pulse to MemClr BAR(SR9-SR16) ShiftRegs to reset to all zeros.
                pause 1                         ' pause 1 milsec
                high NewMin                     ' Set MRCLEAR BAR(SR9-SR16)line high ready for next reset
                
                for b10 = 1 to 4                ' Because the first 4 outputs of SR9 are not connected to leds we
                  pulsout NewSec, 10            ' must send a dummy run of 4 NewSec CLOCK pulses to
                next                            ' clock and shift Shiftregs BAR(SR9-SR16)to start at Led1.(SR9 Output o4)
                                                ' This will also pulse MRCLEAR of CHASE(SR1-SR8) and reset with all zeros ready for new minute.
                if pin4 = 1 then Loop           ' If Sync button still down, loop and check input pins again.
                goto NXT_SEC                    ' else Go start a new Minute



'Synchronization to the second hand is achieved by pressing the reset button
'and then waiting for the second hand to reach 60. The LED display will be blanked and
'then will start from 1 when the button is let go.
'The clock will stay in synch until power is removed.


