'PROGRAM: ADC001.BAS

'BASIC Stamp program that demonstrates
'how to use the serial ADC module.

'Every second the ADC performs a reading
'and transmits the ADC value in variable b0
'to the host computer via the debug command. 

'Connect the ADC module to the Stamp, 
'and then connect a sensor to the ADC.

symbol	ADC_CS	=	0	'ADC chip select pin (output)
symbol	ADC_CLK	=	1	'ADC clock pin (output)
symbol	ADC_D_I	=	pin2	'ADC data pin (input)
symbol 	switch	= 	pin3	'ADC push switch (input)

symbol	data	=	b4	'data byte
symbol	i	=	b5	'scratchpad counter

' Initialise pins

init:	let dirs = %11110011		'pin 2 & 3 input, rest outputs. 
	let pins = %00000001		'ADC_CS high

' Main program loop

loop:	gosub adcread			'Get the ADC reading.
	debug data			'Transmit to computer.
	pause 1000			'Pause for 1 second.
	goto loop			'Do it forever. 

' This sub-procedure gets a reading from the serial ADC unit.
' Data is returned in byte variable 'data'

adcread:
	low ADC_CLK			'Put clock line in starting state. 
	low ADC_CS			'Select ADC. 
	pulsout ADC_CLK, 1		'10 us clock pulse. 
	let data = 0			'Clear data. 
	for i = 1 to 8			'Eight data bits. 
	   let data = data * 2		'Perform shift left. 
	   pulsout ADC_CLK, 1		'10 us clock pulse. 
	   let data = data + ADC_D_I	'Put bit in LSB of data. 
	next i				'Do it again. 
	high ADC_CS			'Deselect ADC when done. 
	return	


