'PROGRAM: ADC005.BAS

'BASIC Stamp program that demonstrates
'how to use the serial ADC module with 
'the analogue multiplexer. Also demonstrates
'how pin3 is used as both input and output
'at different points during the program.

'Every time the ADC push switch is pressed
'the Stamp performs both ADC readings
'and displays the ADC values on the LCD.

'Connect the ADC module to the Stamp, and
'then connect the multiplexer to the ADC.
'Connect a 2mm cable between the multiplexer
'and pin3 on the serial ADC. Connect two sensors
'to the multiplexer.
'Connect the LCD module to the Stamp.

symbol	ADC_CS	=	0	'ADC chip select pin (output)
symbol	ADC_CLK	=	1	'ADC clock pin (output)
symbol	ADC_D_I	=	pin2	'ADC data pin (input)
symbol 	ADC_MPX = 	3	'ADC multiplexer select pin (output)
symbol 	SWITCH	= 	pin3	'ADC push switch (input)

symbol	S_OUT	= 	7	'serial LCD output (output)

symbol	data	=	b4	'data byte
symbol	i	=	b5	'scratchpad counter
symbol 	counter	= 	b10	'counter

init:	let dirs = %11111011	' pin 2 input, rest outputs. 
	let pins = %10000001	' ADC_CS and S_OUT high

	pause 5				'Short pause
	serout S_OUT,T2400,(254,1) 	'Clear lcd
	pause 30			'Short pause.

loop:	low ADC_MPX			'Select sensor.
	pause 30			'Short delay.
	gosub adcread			'Get the ADC reading.
	
	'Print ADC value on first LCD line
	serout S_OUT,T2400,(254,128,"Sensor 1= ",#data,"  ")				

	high ADC_MPX			'Select sensor.
	pause 30			'Short pause.
	gosub adcread			'Get the ADC reading. 

	'Print ADC value on second LCD line
	serout S_OUT,T2400,(254,192,"Sensor 2= ",#data,"  ")

	input	ADC_MPX			'Make pin 3 an input.

inloop:	if SWITCH = 0 then inloop	'Wait until switch is pushed.
loop2:	if SWITCH = 1 then loop2	'Now wait for switch release.

	'Note that it is essential to wait for push switch to be
	'released before moving on. This is because push switch
	'would 'over-ride' the output ADC_MPX setting. 

	output ADC_MPX			'Make pin 3 an output.
	let counter = counter + 1	'Increment counter.
	goto loop			'Loop.


' This sub-procedure gets a reading from the serial ADC unit.
' Data is returned in byte variable 'data'

adcread:
	low ADC_CLK			'Put clock line in starting state. 
	low ADC_CS			'Select ADC. 
	pulsout ADC_CLK, 1		'10 us clock pulse. 
	let data = 0			'Clear data. 
	for i = 1 to 8			'Eight data bits. 
	   let data = data * 2		'Perform shift left. 
	   pulsout ADC_CLK, 1		'10 us clock pulse. 
	   let data = data + ADC_D_I	'Put bit in LSB of data. 
	next i				'Do it again. 
	high ADC_CS			'Deselect ADC when done. 
	return	


