'PROGRAM: DATA001.BAS

'BASIC Stamp program that demonstrates
'how to perform a datalogging experiment.

'The program takes 255 readings for each sensor,
'recording the values on the two pages of the
'datalogger EEPROM memory. The readings are
'monitored by use of the serial LCD module.

'Use program data002.bas to recall the values on an LCD.
'Use program data003.bas to print the values directly.
'Use program data004.bas to save the values in CSV format
'on a computer by using DataLink application.

'Connect the ADC module to the Stamp, connect
'the multiplxer to the ADC, and connect two
'sensors to the multiplexer. Link the multiplexer
'select input to pin3 on the ADC by a 2mm-2mm cable.

'Connect the Datalogger module to the Stamp and
'then connect the serial LCD module to the 
'datalogger module.

'Warning - this program uses almost all of the 
'Stamp memory. If you wish to modify it you can 
'release a lot of space by reducing the number of 
'serout commands for the LCD.

symbol	ADC_CS	=	0	'ADC chip select pin (output)
symbol	ADC_CLK	=	1	'ADC clock pin (output)
symbol	ADC_D_I	=	pin2	'ADC data pin (input)
symbol 	ADC_MPX	= 	3	'ADC multiplexer select pin (output)

symbol 	EE_CS	= 	4	'EEPROM chip select pin (output)
symbol 	EE_D_I	= 	pin5	'EEPROM data pin (when input)
symbol 	EE_D_O 	= 	5	'EEPROM data pin (when output)
symbol 	EE_CLK 	= 	6	'EEPROM clock pin (output)

symbol	S_OUT	= 	7	'serial output (output)

symbol	data	=	b4	'data byte
symbol	i	=	b5	'scratchpad counter
symbol 	ShifReg = 	w3	'scratchpad shift register

symbol  address = 	b8	'EEPROM address 
symbol	page	= 	b9	'EEPROM page
symbol 	clocks 	= 	b11	'scratchpad clock counter

init:	let dirs = %11111011	' pin 2 input, rest outputs. 
	let pins = %10000001	' ADC_CS and S_OUT high

	pause 5				'Short pause
	serout S_OUT,T2400,(254,1) 	'Clear lcd
	pause 30			'Short pause.

main:	for address = 0 to 255		'Start a for..next loop
	   low ADC_MPX			'Select sensor.
	   pause 30			'Short delay.
	   gosub adcread		'Get the ADC reading.
	
	   let page = 0			'Move to page 0.
	   gosub eewrite		'Write to EEPROM.

	   'debug data			'Debug data.
	
	   serout S_OUT,T2400,(254,128,"#",#address,"  ")
	   serout S_OUT,T2400,(254,134,"S1 = ",#data,"  ")
					'Msg on first line	

	   high ADC_MPX			'Select sensor.
	   pause 30			'Short pause.
	   gosub adcread		'Get the ADC reading. 

	   let page = 1			'Move to page 1.
	   gosub eewrite		'Write to EEPROM.

	   'debug data			'Debug data

	   serout S_OUT,T2400,(254,198,"S2 = ",#data,"  ")
					'Msg on second line

	   pause 100			'Short delay.
	next address			'Next in loop. 

	serout S_OUT,T2400,(254,1)	'Clear LCD.
	pause 30			'Short delay.
	serout S_OUT,T2400,("Done!")	'Finished message.

	end				'End program

' This sub-procedure gets a reading from the serial ADC unit.
' Data is returned in byte variable 'data'

adcread:
	low ADC_CLK			'Put clock line in starting state. 
	low ADC_CS			'Select ADC. 
	pulsout ADC_CLK, 1		'10 us clock pulse. 
	let data = 0			'Clear data. 
	for i = 1 to 8			'Eight data bits. 
	   let data = data * 2		'Perform shift left. 
	   pulsout ADC_CLK, 1		'10 us clock pulse. 
	   let data = data + ADC_D_I	'Put bit in LSB of data. 
	next i				'Do it again. 
	high ADC_CS			'Deselect ADC when done. 
	return	

'This sub-procedure writes a byte to the EEPROM.
'data' is written to 'address' on 'page' 

eewrite:
	gosub eenabl			'Enable.
	let ShifReg = $A00 		'Get the write opcode.  
	let ShifReg = ShifReg | w4	'OR in the address bits.
	let clocks = 12			'Send 12 bits to EEPROM. 
	high EE_CS			'Select EEPROM.
	gosub eeout			'Send the opcode/address. 
	let ShifReg = data * 16 	'Move bit 7 to bit 11. 
	let clocks = 8			'Eight data bits. 
	gosub eeout			'Send the data.
	low EE_CS			'Deselect EEPROM.
	gosub edisbl			'Write Protect.
	return

' EEPROM sub-procedures

Eenabl:	let ShifReg = $980		'Get the write-enable opcode.
	high EE_CS			'Chip select on. 
	let clocks = 12			'Send 12 bits to EEPROM. 
	gosub eeout			'Send the opcode. 
	low EE_CS			'Deselect the EEPROM. 
	return

Edisbl:	let ShifReg = $800		'Get the write-disable opcode. 
	high EE_CS			'Chip select on. 
	let clocks = 12			'Send 12 bits to EEPROM. 
	gosub eeout			'Send the opcode.
	low EE_CS			'Deselect the EEPROM.
	return

eein:	input EE_D_O			'Change the data line to input.
	let data = 0			'Clear data byte.
	for i = 1 to 8			'Prepare to get 8 bits. 
           let data = data * 2		'Shift EEdata to the left. 
           high EE_CLK			'Data valid on rising edge.
           let data = data + EE_D_I 	'Move data to lsb of variable. 
           low EE_CLK			'End of clock pulse. 
	next i				'Get another bit.
	output EE_D_O			'Restore data line to output. 
	return

eeout:	for i = 1 to clocks		'Number of bits to shift out. 
           let EE_D_I = ShifReg / $800 	'Get bit 12 of ShifReg.
	   pulsout EE_CLK,10		'Output a brief clock pulse. 
	   let ShifReg = ShifReg * 2 	'Shift register to the left. 
	next i 				'Send another bit. 
	return

