'PROGRAM: DATA002.BAS

'BASIC Stamp program that demonstrates
'how to retrieve datalogging data by 
'displaying on the serial LCD.

'The program recalls the data saved by 
'program data001.bas. Each reading is 
'retrieved in turn when the switch on 
'the ADC module is pushed.

'Connect the ADC module to the Stamp.
'Connect the Datalogger module to the Stamp
'and then connect the serial LCD module to 
'the datalogger module.

symbol 	switch	= 	pin3	'ADC input switch (input)

symbol 	EE_CS	= 	4	'EEPROM chip select pin (output)
symbol 	EE_D_I	= 	pin5	'EEPROM data pin (when input)
symbol 	EE_D_O 	= 	5	'EEPROM data pin (when output)
symbol 	EE_CLK 	= 	6	'EEPROM clock pin (output)

symbol	S_OUT	= 	7	'serial output (output)

symbol	data	=	b4	'data byte
symbol	i	=	b5	'scratchpad counter
symbol 	ShifReg = 	w3	'scratchpad shift register

symbol  address = 	b8	'EEPROM address 
symbol	page	= 	b9	'EEPROM page
symbol 	clocks 	= 	b11	'scratchpad clock counter

init:	let dirs = %11110011	' pin 2&3 input, rest outputs. 
	let pins = %10000001	' ADC_CS and S_OUT high

	pause 5				'Short pause
	serout S_OUT,T2400,(254,1) 	'Clear lcd
	pause 30			'Short pause.

main:	let page = 0			'Move to page 0.
	gosub eeread			'Read data from EEPROM.

	'Debug data			'Debug  data.
	
	serout S_OUT,T2400,(254,128,"#",#address,"  ")
	serout S_OUT,T2400,(254,134,"S1 = ",#data,"  ")
					'Msg on first line	
   	
	let page = 1			'Move to page 1.
	gosub eeread			'Read data from EEPROM.
	 
	'Debug data			'Debug data

	serout S_OUT,T2400,(254,198,"S2 = ",#data,"  ")
					'Msg on second line

inloop:	if switch = 0 then inloop	'Wait for switch push.
	
	let address = address + 1	'Increment address.
	pause 50			'Short debounce time.
	
	goto main			'Loop.

'This sub-procedure reads a byte from the EEPROM.
'data' is read from 'address' on 'page' 

eeread:	
	let ShifReg = $C00		'Get the read opcode.
	let ShifReg = ShifReg | w4	'OR in the address bits.
	let clocks = 12			'Send 12 bits to EEPROM. 
	high EE_CS			'Chip select on. 
	gosub eeout			'Send the opcode/address. 
	gosub eein			'Receive the byte.
	low EE_CS			'Deselect the EEPROM. 
	return

' EEPROM sub-procedures

eein:	input EE_D_O			'Change the data line to input.
	let data = 0			'Clear data byte.
	for i = 1 to 8			'Prepare to get 8 bits. 
           let data = data * 2		'Shift EEdata to the left. 
           high EE_CLK			'Data valid on rising edge.
           let data = data + EE_D_I 	'Move data to lsb of variable. 
           low EE_CLK			'End of clock pulse. 
	next i				'Get another bit.
	output EE_D_O			'Restore data line to output. 
	return

eeout:	for i = 1 to clocks		'Number of bits to shift out. 
           let EE_D_I = ShifReg / $800 	'Get bit 12 of W1.
	   pulsout EE_CLK,10		'Output a brief clock pulse. 
	   let ShifReg = ShifReg * 2 	'Shift register to the left. 
	next i 				'Send another bit. 
	return

