'PROGRAM: DATA003.BAS

'BASIC Stamp program that demonstrates
'how to retrieve datalogging data by
'printing it directly (without computer).

'The program recalls the data saved by 
'program data001.bas. Each reading is 
'printed via the Printer module.

'To save paper this example only prints
'the first 25 readings! The program can take
'up to ten seconds before printing if using
'a laserjet or inkjet printer.

'Connect the Datalogger module to the Stamp
'and then connect the serial printer module 
'to the datalogger module.

symbol 	EE_CS	= 	4	'EEPROM chip select pin (output)
symbol 	EE_D_I	= 	pin5	'EEPROM data pin (when input)
symbol 	EE_D_O 	= 	5	'EEPROM data pin (when output)
symbol 	EE_CLK 	= 	6	'EEPROM clock pin (output)

symbol	S_OUT	= 	7	'serial output (output)

symbol	data	=	b4	'data byte
symbol	i	=	b5	'scratchpad counter
symbol 	ShifReg = 	w3	'scratchpad shift register

symbol  address = 	b8	'EEPROM address 
symbol	page	= 	b9	'EEPROM page
symbol 	clocks 	= 	b11	'scratchpad clock counter

'Define useful ASCII characters
symbol	STX	=	2		'Start of text ("wake-up")
symbol	ETX	=	3		'End of text ("sleep")
symbol	TAB	=	9		'tab
symbol	LF	=	10		'line feed
symbol	FF	=	12		'form feed

init:	let dirs = %11111011	'Pin 2 input, rest outputs. 
	let pins = %10000001	'S_OUT high
	pause 5			'Short pause

	'Print titles
	serout S_OUT,T2400,(STX)	'Wake up printer module.
	serout S_OUT,T2400,(TAB,"Address",TAB,TAB,"Sen 1:",TAB,TAB,"Sen 2:",CR,LF,LF)

main:	for address = 0 to 25		'Start a for..next loop.
	   let page = 0			'Move to page 0.
	   gosub eeread			'Read data from EEPROM.
	   serout S_OUT,T2400,(TAB,#address,TAB,TAB,#data)
  	
	   let page = 1			'Move to page 1.
	   gosub eeread			'Read data from EEPROM.
	   serout S_OUT,T2400,(TAB,TAB,#data,CR,LF)
	next address			'Next loop.
  		
	serout S_OUT,T2400,(FF)		'Print form-feed
	serout S_OUT,T2400,(ETX)	'Send printer module to sleep
	end				'end 


'This sub-procedure reads a byte from the EEPROM.
'data' is read from 'address' on 'page' 

eeread:	
	let ShifReg = $C00		'Get the read opcode.
	let ShifReg = ShifReg | w4	'OR in the address bits.
	let clocks = 12			'Send 12 bits to EEPROM. 
	high EE_CS			'Chip select on. 
	gosub eeout			'Send the opcode/address. 
	gosub eein			'Receive the byte.
	low EE_CS			'Deselect the EEPROM. 
	return

' EEPROM sub-procedures

eein:	input EE_D_O			'Change the data line to input.
	let data = 0			'Clear data byte.
	for i = 1 to 8			'Prepare to get 8 bits. 
           let data = data * 2		'Shift EEdata to the left. 
           high EE_CLK			'Data valid on rising edge.
           let data = data + EE_D_I 	'Move data to lsb of variable. 
           low EE_CLK			'End of clock pulse. 
	next i				'Get another bit.
	output EE_D_O			'Restore data line to output. 
	return

eeout:	for i = 1 to clocks		'Number of bits to shift out. 
           let EE_D_I = ShifReg / $800 	'Get bit 12 of W1.
	   pulsout EE_CLK,10		'Output a brief clock pulse. 
	   let ShifReg = ShifReg * 2 	'Shift register to the left. 
	next i 				'Send another bit. 
	return

