'PROGRAM: LCD001.BAS

'BASIC Stamp program that demonstrates
'how to use the serial LCD Module.

'Connect the LCD module to the Stamp.
'It may be necessary to adjust the contrast
'via the preset resistor on the LCD module.

symbol	S_OUT	= 	7		'serial LCD output (output)

symbol	counter	= 	b4		'counter
symbol	i	= 	b5		'scratchpad counter

'LCD commands. To transmit a command (rather than a character)
'it must be preceeded by the command prefix (254).

symbol	lcd_clr	=	1		'clear display
symbol	lcd_l1	=	128		'move to line 1
symbol	lcd_l2	=	192		'move to line 2
symbol	lcd_off	=	8		'hide display
symbol	lcd_on	=	12		'restore display

init:	'Make serial pin output, and leave high for 5ms
	'This provides time for LCD serial input to reset. 

	high S_OUT			'Make pin7 high output				
	pause 5				'Short pause.

	'Now blank LCD by sending clear command.
	'Note that the clear command must ALWAYS be
	'followed by at least a 30 ms delay.

	serout S_OUT,T2400,(254,lcd_clr)	'Clear LCD.
	pause 30				'Short pause.


loop:	'Move to start of line 1 by sending line1 command.
	serout S_OUT,T2400,(254,lcd_l1)

	'Display message
	serout S_OUT,T2400,("Counter= ",#counter,"  ")				

	'Note the # before the variable. This ensures
	'variable numbers are transmitted as a string of
	'ASCII characters rather than as a single byte.

	'Also note the two extra spaces following the message.	
	'This 'blanks' the last 2 digits when a 1 or 2 digit
	'number is written over the top of a 3-digit number.

	'Move to start of line 2 and write 'Hello!'

	serout S_OUT,T2400,(254,lcd_l2,"Hello!")
	pause 2000				'Wait 2 seconds.


	'Now flash display on and off 5 times

	for i = 1 to 5				'Start for.. loop.
	   serout S_OUT,T2400,(254,lcd_off)	'Hide lcd display.
	   pause 500				'Short delay.
	   serout S_OUT,T2400,(254,lcd_on)	'Restore display.
	   pause 500				'Short delay.
	next i					'Next for.. loop.


	let counter = counter + 1		'Increment counter.
	goto loop				'Loop.	


