'PROGRAM: LOG001.BAS

'BASIC Stamp program that demonstrates
'how to use the Datalogger EEPROM.

'This program saves the values 1-255 into
'the EEPROM memory, and then reads the values
'and transmits them via the debug command.

'If operating correctly the value b0 (data)
'should descend from 255 to 0. If datalogger
'is not connected b0 will stay at 255.

'Connect the Datalogger module to the Stamp. 

symbol 	EE_CS	= 	4	'EEPROM chip select pin (output)
symbol 	EE_D_I	= 	pin5	'EEPROM data pin (when input)
symbol 	EE_D_O 	= 	5	'EEPROM data pin (when output)
symbol 	EE_CLK 	= 	6	'EEPROM clock pin (output)

symbol	data	=	b4	'data byte
symbol	i	=	b5	'scratchpad counter
symbol 	ShifReg = 	w3	'scratchpad shift register

symbol  address = 	b8	'EEPROM address 
symbol	page	= 	b9	'EEPROM page

symbol	j	= 	b10
symbol 	clocks 	= 	b11	'scratchpad clock counter

init:	let dirs = %11111011	' pin 2 input, rest outputs. 

	let address = 0			'Set address to 0.

	for j = 0 to 255
	   let address = j
	   let data = 255 - j
	   gosub eewrite		'Write to EEPROM.
	
	   let address = j		'Reset address.
	   gosub eeread			'Read from EEPROM.
	   debug data			'Transmit value
	   pause 250	
	next j	


	end

'This sub-procedure writes a byte to the EEPROM.
'data' is written to 'address' on 'page' 

eewrite:
	gosub eenabl			'Enable.
	let ShifReg = $A00 		'Get the write opcode.  
	let ShifReg = ShifReg | w4	'OR in the address bits.
	let clocks = 12			'Send 12 bits to EEPROM. 
	high EE_CS			'Select EEPROM.
	gosub eeout			'Send the opcode/address. 
	let ShifReg = data * 16 	'Move bit 7 to bit 11. 
	let clocks = 8			'Eight data bits. 
	gosub eeout			'Send the data.
	low EE_CS			'Deselect EEPROM.
	gosub edisbl			'Write Protect.
	return

'This sub-procedure reads a byte from the EEPROM.
'data' is read from 'address' on 'page' 

eeread:	
	let ShifReg = $C00		'Get the read opcode.
	let ShifReg = ShifReg | w4	'OR in the address bits.
	let clocks = 12			'Send 12 bits to EEPROM. 
	high EE_CS			'Chip select on. 
	gosub eeout			'Send the opcode/address. 
	gosub eein			'Receive the byte.
	low EE_CS			'Deselect the EEPROM. 
	return

' EEPROM sub-procedures

Eenabl:	let ShifReg = $980		'Get the write-enable opcode.
	high EE_CS			'Chip select on. 
	let clocks = 12			'Send 12 bits to EEPROM. 
	gosub eeout			'Send the opcode. 
	low EE_CS			'Deselect the EEPROM. 
	return

Edisbl:	let ShifReg = $800		'Get the write-disable opcode. 
	high EE_CS			'Chip select on. 
	let clocks = 12			'Send 12 bits to EEPROM. 
	gosub eeout			'Send the opcode.
	low EE_CS			'Deselect the EEPROM.
	return

eein:	input EE_D_O			'Change the data line to input.
	let data = 0			'Clear data byte.
	for i = 1 to 8			'Prepare to get 8 bits. 
           let data = data * 2		'Shift EEdata to the left. 
           high EE_CLK			'Data valid on rising edge.
           let data = data + EE_D_I 	'Move data to lsb of variable. 
           low EE_CLK			'End of clock pulse. 
	next i				'Get another bit.
	output EE_D_O			'Restore data line to output. 
	return

eeout:	for i = 1 to clocks		'Number of bits to shift out. 
           let EE_D_I = ShifReg / $800 	'Get bit 12 of ShifReg.
	   pulsout EE_CLK,10		'Output a brief clock pulse. 
	   let ShifReg = ShifReg * 2 	'Shift register to the left. 
	next i 				'Send another bit. 
	return

