; How to upgrade the VMUSIC2/VDRIVE2 firmware

; 1) Download the appropriate FTDI firmware "FTD" (not "ROM") upgrade file (e.g. version 3.55)
; 	VDRIVE2 uses VDAP firmware e.g. file called ftrfb_main_3_55_VDPAB.ftd
;	VMUSIC2 uses VMSC firmware e.g. file called ftrfb_main_3_55_VMSC1B.ftd
; 2) Rename the file to ftrfb.ftd and save onto the USB thumb drive
; 3) Carefully open the plastic case of the VDRIVE2/VMUSIC2 and locate the jumper CN4
;    next to the USB socket. Move the jumper to the prog position (next to the 'CN4' text) 
; 3) Ensure the spi/serial jumper CN3 is in the (default outer) serial position
; 4) Connect your PICAXE-28X1 hardware as follows:
; 	VMUSIC2/VDRIVE2		PICAXE-28X1
; 	1 Black	GND	-	0V
; 	2 Brown	RTS	-	not connected
; 	3 Red		V+	-	V+
; 	4 Orange 	RXD	- 	HTXD	input6	C6
; 	5 Yellow	TXD	- 	HRXD	input7	C7
; 	6 Green	CTS	-	tie to 0V
; 	7 
; 	8 Blue	RI	-	not connected
; 	Note RXD on VMUSIC2 connects to TXD on PICAXE etc.
; 5) Insert the USB thumb drive into the module and then power up the system.
; 6) Download and run the following PICAXE program.
; 7) After the upgrade is complete, 
;    move the programming jumper CN4 back to the original position.

; set picaxe type
#picaxe 28x1

; set COM port used for download
;#com 4

; open terminal after download
; This is to view the 'sertxd' debugging comments
#terminal 4800


symbol first_byte = b0
symbol point = b1
symbol temp = b2
symbol loopcounter = b3

setup:
	; setup serial hardware 
	; at 9600 with background receive
	hsersetup b9600_4,%01
	low 0		; ensure CTS is low
	
init:
	; Send Es until the unit responds correctly
	sertxd ("<Sent> E",CR,LF)
	hserout 0,("E",CR)
	gosub	get_response
	if first_byte <> "E" then init

main:
	; check to see if a drive is actually inserted
	; response will start D for yes and N for no
	sertxd ("<Sent> Check Drive",CR,LF)
	hserout 0,(CR)
	gosub	get_response
	if first_byte <> "D" then main 
	
	; Download the firmware
	sertxd ("<Sent> Firmware upgrade started",CR,LF)
	hserout 0,("fwu ftrfb.ftd",CR)
	stop
	
	; Sub procedure to receive background bytes
get_response:
	pause 1000			; wait a while	
	sertxd ("<Response> ")
	
	point = 0			; reset local pointer
	
	get point,first_byte	; Save the first reply byte
	
	do
	   get point,temp		; get returned byte
	   sertxd (temp)		; transmit it
	   inc point		; increment pointer
	loop while temp <> CR	; if not CR loop
	
	sertxd (LF)			; Add a LF to the received CR
	sertxd (CR,LF)		; Do another blank line

	hserptr = 0			; reset the background receive pointer 
	return

